use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;

set_spell_cmd('hunspell -l');
add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
ASSAD
Andy
Anwar
Arhuman
Arnaud
Baugh
Bircumshaw
Bloch
CPAN
Drive
Emmanuel
Google
Håkon
Hægland
Item
Maischein
Matthias
Max
Mike
Mohammad
Mullin
Net
Nicolas
Peter
Rinaldo
Rodriguez
Sawyer
Schilli
Scott
Simple
Tim
Todd
Yxes
arhuman
atoomic
corion
cpan
emmanuel
github
hakon
lib
matthias
mohammad
perlmeister
peter
thomas
tim
toddr
wells
xsawyerx
